package com.example;

import java.io.File;
import java.util.Map;
import java.util.HashMap;
import java.util.Scanner;
import java.util.ServiceLoader;
import java.net.MalformedURLException;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

import org.osgi.framework.launch.Framework;
import org.osgi.framework.launch.FrameworkFactory;

public class FrameworkService
{
	private    Framework framework = null;

	private final  String    cache =  "bundles-cache";
	private final  String[]  files = {"bundles/org.apache.felix.eventadmin-1.4.8.jar",
			                          "bundles/bundle-exchange-1.0.0.jar",
			                          "bundles/bundle-publisher-1.0.0.jar"};
	private final String  BUNDLE_PUBLISHER = "com.osgi.bundle.bundle-publisher";
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public FrameworkService()
	{
		createFramework();
        if (framework == null) {
            System.out.println ("framework is not created");
        	return;
        }
        try {
	        try {
		    	installBundles();
		    	startBundles  ();
				//------------------------
		        // создание объекта чтения из стандартного потока ввода
		        Scanner in = new Scanner(System.in);
		        // чтение строки из консоли
		        System.out.print ("\n\nEnter pub : " );
		        String cmd = in.nextLine();
		        while (!cmd.equalsIgnoreCase("pub")) {
		            cmd = in.nextLine();
		        }
				//------------------------
				for (int i = 0; i < framework.getBundleContext().getBundles().length; i++) {
		        	Bundle bundle = framework.getBundleContext().getBundles()[i];
		        	if (bundle.getSymbolicName().equalsIgnoreCase(BUNDLE_PUBLISHER)) {
						try {
							bundle.start();
							printBundleParams(bundle, "Bundle", false);
						} catch (BundleException e) {
							System.err.println("0. BundleException : " + e.getMessage());
						}
		        	}
		        }
				//------------------------
		        System.out.print ("\n\nEnter exit : " );

		        // чтение строки из консоли
		        while (!cmd.equalsIgnoreCase("exit"))
		            cmd = in.nextLine();
		        in.close();
				//------------------------
	            stopFramework();
				//------------------------
			} catch (Exception e) {
				System.err.println("Exception : " + e.getMessage());
			}
        } finally { 
        	if (framework != null) {
        		try {
        			framework.stop();
        			framework.waitForStop(2000);
				} catch (BundleException e) {
			        System.err.println ("BundleException : " + e.getMessage());
				} catch (InterruptedException e) {
			        System.err.println ("InterruptedException : " + e.getMessage());
				}
        	}
        } 
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void createFramework()
	{
		ServiceLoader<FrameworkFactory> serviceLoader = ServiceLoader.load(FrameworkFactory.class);
    	
    	FrameworkFactory factory = serviceLoader.iterator().next();
    	try {
			if (factory != null) {
				Map<String, String> props = new HashMap<String, String>();
				props.put("org.osgi.framework.storage", cache); 
				props.put("org.osgi.framework.storage.clean", "onFirstInit");
				framework = factory.newFramework(props);
			}
    
			if (framework == null) {
				System.out.println ("Framework is not created");
			} else {
				System.out.println ("Framework is created");
				printBundleParams(framework, "Framework", true);
	        	framework.init ();
				printBundleState(framework, "Framework");
				framework.start();
				printBundleState(framework, "Framework");
			}
    	} catch(Exception e){
			System.err.println ("CreateFramework Exception : " + e.getMessage());
    	}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void stopFramework() throws BundleException, InterruptedException
	{ 
		framework.stop(); 
		framework.waitForStop(10000);
		System.out.println("stop : framework");
	} 
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void installBundles()
	{
		System.out.println ("\nINSTALL BUNDLES ...\n" );
		BundleContext bctx = framework.getBundleContext();
		String MSG_TEMPL = "Bundle <%s>";
		try {
	        for (int i = 0; i < files.length; i++) {
				File file = new File (files[i]);
				if (file.exists()) {
				    String location = file.toURI().toURL().toString();
				    Bundle bundle = bctx.installBundle(location);
					if (bundle != null) {
						String msg;
						msg = String.format(MSG_TEMPL, file.getName());
						System.out.println (msg);
						printBundleParams(bundle, "Bundle", true);
					}
				}
	        }
		} catch (MalformedURLException e) {
			System.err.println(e.getMessage());
		} catch (BundleException e) {
			System.err.println(e.getMessage());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void startBundles()
	{
        System.out.println("\nSTART BUNDLES\n");
		BundleContext bctx = framework.getBundleContext();

        Bundle[] bundles = bctx.getBundles();
       	for (int b = 1; b < bundles.length; b++){
       		Bundle bndl = bundles[b];
			try {
				if (!bndl.getSymbolicName().equals(BUNDLE_PUBLISHER)) {
					bndl.start();
					printBundleParams(bndl, "Bundle", true);
				}
			} catch (BundleException e) {
				System.err.println(e.getMessage());
			}
       	}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void printBundleState(Bundle bundle, final String prefix)
	{
		if (bundle == null)
			return;
		switch (bundle.getState()) {		
		case Bundle.UNINSTALLED :  // 0x00000001
			System.out.println (prefix + " state : UNINSTALLED");
			break;
		case Bundle.INSTALLED :    // 0x00000002
			System.out.println (prefix + " state : INSTALLED");
			break;
		case Bundle.RESOLVED :     // 0x00000004
			System.out.println (prefix + " state : RESOLVED");
			break;			
		case Bundle.STARTING :     // 0x00000008
			System.out.println (prefix + " state : STARTING");
			break;
		case Bundle.STOPPING :     // 0x00000010
			System.out.println (prefix + " state : STOPPING");
			break;
		case Bundle.ACTIVE :       // 0x00000020
			System.out.println (prefix + " state : ACTIVE");
			break;
		default :
			System.out.println (prefix + " state.code = " + bundle.getState());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void printBundleParams(Bundle bundle, final String prefix, final boolean full)
	{
		if (full) {
			System.out.println ("   Version = " + bundle.getVersion());
			System.out.println ("   SymbolicName = " + bundle.getSymbolicName());
			System.out.println ("   Location = " + bundle.getLocation());
			System.out.println ("   BundleId = " + bundle.getBundleId());
		}
		printBundleState(bundle, "   " + prefix);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    public static void main( String[] args )
    {
        new FrameworkService();
        System.exit(0);
    }
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
